/***************************************************************************
 *   Copyright (C) 2005 by Peter Müller   *
 *   pmueller@cs.tu-berlin.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "explosion.h"

Explosion::Explosion(GLuint decal_tex, GLuint particle_texture, 
					 const Vec4& start_color, const Vec4& end_color,
                     const Vec3& p, const Vec3 & normal, float size, bool _decal, 
                     float time)
: 
Effect(p),
decal(NULL),
psystem(NULL)
{
	const float etime = 1.0f;
	
	if (_decal) {
		decal = new TimeLimitDecal(decal_tex, p, true, normal, Vec2(size*1,size*1), Vec2(size*3,size*3), 
									Vec4(1.0f, 1.0f, 1.0f, 1.0f), time, etime*0.75f, 0.00f, etime);
	}
	else {
		decal = NULL;
	}
	
	psystem = new ParticleSystem(particle_texture, p, 300, 300, etime/2, etime, false, 0.0f,
								Vec3::ZERO, size/etime, size*2/etime, true, size/4, start_color,
								end_color, time);
}


Explosion::~Explosion()
{
	if (decal != NULL)
		delete decal;
	
	delete psystem;
}

void Explosion::process(float time) {
	if (decal != NULL) 
		decal->process(time);
	
	psystem->process(time);
}

void Explosion::render(const Vec3 & campos) {
	if (decal != NULL) 
		decal->render(campos);
	
	psystem->render(campos);
}

void Explosion::stop() {
	if (decal != NULL) 
		decal->stop();
	
	psystem->stop();
}

bool Explosion::done() {
	if (decal != NULL) 
		return decal->done() && psystem->done();
	else
		return psystem->done();
}

void Explosion::setPos(const Vec3& _pos) {
	pos = _pos;
	
	if (decal != NULL) 
		decal->setPos(_pos);
	psystem->setPos(_pos);
}
