/***************************************************************************
 *   Copyright (C) 2005 by Peter Müller   *
 *   pmueller@cs.tu-berlin.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef PARTICLESYSTEM_H
#define PARTICLESYSTEM_H

#include <algorithm>

#define GL_GLEXT_PROTOTYPES
#include <GL/gl.h>
#include <GL/glu.h>

#include "effect.h"
#include "cometen_math.h"

using namespace std;

/**
@author Peter Müller
*/


class ParticleSystem : public Effect {
private:	
	class Particle {
	public:
		Vec3 pos;
		Vec3 vel;
		Vec4 color; // temp for rendering
		float start_time;
		float life_time;
		bool active;
	};
	
	Vec4 start_color;
	Vec4 end_color;
	
	Vec3 velocity;
	float minvar, maxvar;
	bool slowdown;
	
	float size;
	
	float minlife, maxlife;
	float new_per_sec;
	bool create_new;
	float new_particle;
		
	float last_create;
	float last_update;
	
	Particle * particle_arr;
	int * index_arr;
	int max_particle;
	int num_active;
	GLuint texture;
	
public:
	ParticleSystem(GLuint texture, 
	               const Vec3 & pos,
	               int max_particle,
	               int create_at_start,
	               float minlife, float maxlife, 
				   bool create_new, 
	               float new_per_sec,
	               const Vec3 & velocity, 
	               float minvar, float maxvar,
	               bool slowdown,
	               float size, 
	               const Vec4 & start_color, 
	               const Vec4 & end_color,
	               float time);
	~ParticleSystem();
	
	virtual void process(float time);
	virtual void render(const Vec3 & campos);
	virtual void stop();
	virtual bool done();
	
	// set / get
};



#endif
